 function [fancy,param,ster,robster,innovations, HT, SBICcrit]=fitARMAGARCH0(data,x,model,errortype,startingvals)
%
% Purpose:     Fit ARMA(r,s)-GJR-GARCH(p,o,q) models to data (column)
%
% Syntax:
% [fancy,param,ster,robster,innovations,SBIC]=fitARMAGARCH(data,model,errortype)
%
% Inputs:
% data         tx1 input date vector
% model        vector in the form [r,s,p,o,q] determining the
%              ARMA(r,s)-GJR-GARCH(p,o,q) model which should be fitted
% errortype    distribution of errors used: 'NORMAL'(default),'STUDENTST',
% (optional)   'GED','SKEWT'
% startingvals (optional), starting values for optimization (e.g. true
%               parameters, if known), must correspond to input 'model'
%
% Outputs:
% fancy        Output cell array displaying the model parameters, their
%              standard errors, z-statistics and corresponding p-values
% param        corresponding parameter vector in the form
%              [phi0 phi1 ... phir theta1 ... thetas omega alpha1 ...
%              alphap gamma1 ... gammas beta1 ... betaq]
% ster         parameter standard errors
% robster      parameter robust standard errors
% innovations  tx1 vector of inovations 
% SBIC         2x1 vector,(1) SBIC of the ARMA model (2) SBIC of the GARCH
%              model
% Input Check
if nargin<4 || isempty(startingvals)
    startingvals=[];
end
if nargin<3 || isempty(errortype)
    errortype='NORMAL';
end

if size(data,2) > 1;
    warning('Data must be a tx1 vector. Transposing data.')
    data=data';
end

% end Input Check

r=model(1);s=model(2);p=model(3);o=model(4);q=model(5);
warning off;

% SBIC
[datasize ~]=size(data);
SBICcrit=zeros(2,1);

% Conditional mean - ARMA(r,s)
options=optimset('fminunc');options.Display='off';

% EXOG REG SEE MFE DOC P. 15  FOR AR(1)
[yadj, ~] = newlagmatrix(data,1,0);
[~, xlags] = newlagmatrix(x,1,0);

if isempty(startingvals)
    [PARAMETERS, LL, ERRORS, SEREGRESSION, DIAGNOSTICS, VCVROBUST, VCV, LIKELIHOOD]=armaxfilter(yadj,1,1:r,1:s,xlags,[],options,[],[]);
else
    [PARAMETERS, LL, ERRORS, SEREGRESSION, DIAGNOSTICS, VCVROBUST, VCV, LIKELIHOOD]=armaxfilter(yadj,1,1:r,1:s,xlags,(startingvals(1:(1+r+s)))',options,[],[]);
end
%output parameters
param=PARAMETERS';robster=sqrt(diag(VCVROBUST))';ster=sqrt(diag(VCV)');
SBICcrit(1)=SBIC(datasize,length(PARAMETERS),LL);

% Conditional variance - GJR-GARCH(p,o,q)
if isempty(startingvals)
    [PARAMETERS,LL,HT,VCVROBUST,VCV,SCORES,DIAGNOSTICS]=tarch(ERRORS,p,o,q,errortype,2,[],options);
else
    [PARAMETERS,LL,HT,VCVROBUST,VCV,SCORES,DIAGNOSTICS]=tarch(ERRORS,p,o,q,errortype,2,(startingvals((2+r+s):end))',options);
end
% update output parameters
param=[param,PARAMETERS'];robster=[robster,sqrt(diag(VCVROBUST)')];
ster=[ster,sqrt(diag(VCV)')]; innovations=ERRORS./sqrt(HT);
SBICcrit(2)=SBIC(datasize,length(PARAMETERS),LL);

% add fancy output
fancy=cell(5,sum(model)+4);

% first line are parameter names
fancy=cell(3,sum(model)+4);
fancy{1,1}='phi0';
for i=1:model(1)
    fancy{1,1+i}=['phi',num2str(i)];
end
if isempty(i)
    i=0;
end
for j=1:model(2)
    fancy{1,1+i+j}=['theta',num2str(j)];
end
if isempty(j)
    j=0;
end
fancy{1,2+i+j}='omega';
for k=1:model(3)
    fancy{1,2+i+j+k}=['alpha',num2str(k)];
end
if isempty(k)
    k=0;
end
for l=1:model(4)
    fancy{1,2+i+j+k+l}=['gamma',num2str(l)];
end
if isempty(l)
    l=0;
end
for m=1:model(5)
    fancy{1,2+i+j+k+l+m}=['beta',num2str(m)];
end
if isempty(m)
    m=0;
end
fancy{1,3+i+j+k+l+m}='nu';fancy{1,4+i+j+k+l+m}='lambda';

% second line are parameter values, third are standard errors, 4th
% z-statistics and 5th corresponding p-values

for i=1:length(param)
    fancy{2,i}=param(i);
    fancy{3,i}=ster(i);
    fancy{4,i}=param(i)/ster(i);
    fancy{5,i}=1-normcdf(abs(param(i)/ster(i)));
         
warning on
end



