function [reject, pVal, crit,Jstat] = JTest(ghat,T,B,W0,Sigma0,G0,seed)
%{
Function to compute the J-test of overidentifying restrictions used for the
model specifications (see Patton, Oh p. 693).

INPUT:  ghat = difference of moment conditions (m_T - m_S(theta))
        k = # of simulations is k*T
        T = sample size of data
        B = # of bootstrap draws (not replications!)
        W0 = weight matrix
        Sigma0 = asymptotic covariance matrix of moment conditions
        G0 = numerical derivatives estimator of moment conditions
        seed = rng seed
OUTPUTS: reject = dummy if H0 is rejected (1 = rejected)
         pVal = p-value
         crit = critical value
         Jstat = test statistic

This code was developed for the master's thesis:

Valet, Sebastian, 2019, "Estimating Factor Copula Models with Covariates".

It expands the complementary code of: 
 Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
 for Copula-Based Multivariate Models," Journal of the American
 Statistical Association, 108(502), 689-700.

Sebastian Valet
April 2019
%}

% Calculate J-Statistics
    Jstat = T*(ghat'*W0*ghat);

%Calculate A0hat
    R0 = eye(size(Sigma0))-(((Sigma0^(-1/2))*G0)/(G0'*W0*G0))*(G0'*W0*(Sigma0^(1/2)));
    A0hat = W0^(1/2)*Sigma0^(1/2)*R0;
    
% B simulations from N(0,I)
    [mysize,~]=size(A0hat);
    rng(100*seed); u=randn(mysize,B);

% Corresponding u'A0'A0u (1x1) values
    JtsBoot=zeros(B,1);
    for ii=1:B
        JtsBoot(ii,1)=u(:,ii)'*(A0hat')*A0hat*u(:,ii);
    end

% Calculate p-value and critical value
    pVal = sum(JtsBoot>Jstat)/B;
    sortJtsBoot=sort(JtsBoot);
    myelement=0.95*B;
    crit=sortJtsBoot(myelement);

% Rejection dummy
    reject=0;
    if Jstat>crit
        reject=1;
    end